import logging
import time
from typing import Optional

import typer

from . import __version__
from .mqtt_task import mqttc_setup, mqttc_publish, mqttc_close

# config logging in main
LOG_SIMPLE = "%(asctime)s | %(levelname).1s | %(name)s | %(message)s"
LOG_PROCESS = "%(asctime)s | %(levelname).1s | %(processName)s | %(name)s | %(message)s"
LOG_THREAD = "%(asctime)s | %(levelname).1s | %(threadName)s | %(name)s | %(message)s"
logging.basicConfig(
    level="INFO",
    format=LOG_THREAD,
)
# get logger
log = logging.getLogger(__name__)

app = typer.Typer()


def version_callback(value: bool):
    if value:
        typer.secho(f"CLI Version: {__version__}", fg=typer.colors.MAGENTA)
        raise typer.Exit()


# Main command
@app.command()
def main(
    _version: Optional[bool] = typer.Option(
        None, "--version", "-v", callback=version_callback, is_eager=True
    ),
    log_level: str = typer.Option("INFO", envvar="LOG_LEVEL", help="log level"),
    mqtt_broker: str = typer.Option(
        ..., "--broker", "-b", envvar="MQTT_BROKER", help="The Hostname/IP of MQTT broker"
    ),
    mqtt_port: int = typer.Option(1883, "--port", "-p", envvar="MQTT_PORT", help="The port of MQTT broker"),
    mqtt_username: str = typer.Option(
        "", "--username", "-u", envvar="MQTT_USERNAME", help="The name of MQTT user"
    ),
    # mqtt_password: str = typer.Option("", "--password", envvar="MQTT_PASSWORD", prompt=True, hide_input=True),
    mqtt_password: str = typer.Option(
        "", "--password", envvar="MQTT_PASSWORD", help="The password of MQTT user"
    ),
    mqtt_qos: int = typer.Option(0, "--qos", "-q", envvar="MQTT_QOS", help="QoS value for MQTT"),
    mqtt_debug: bool = typer.Option(False, "--debug", "-d", help="debug mqtt"),
):
    """
    A CLI app to handle mqtt
    """
    typer.echo(f"{__name__} starting...")
    if log_level:
        log.setLevel(log_level)
        typer.echo(f"set logging level to {log_level}")
    log.info(f"{__name__} started.")
    log.debug(f"mqtt_broker = {mqtt_broker}")
    log.debug(f"mqtt_port = {mqtt_port}")
    if mqtt_username:
        log.debug(f"mqtt_username = {mqtt_username}")
    log.debug(f"mqtt_debug = {mqtt_debug}")

    # Setup mqtt
    mqttc_setup(mqtt_broker, mqtt_port, mqtt_username, mqtt_password, mqtt_qos)

    # main loop
    run_flag = True
    interval_s = 3.0
    last_check = 0
    while run_flag:
        now = time.time()
        if last_check == 0:
            last_check = now
            continue
        # routine check
        if now > last_check + interval_s:
            mqttc_publish("ysdtest/1", f"hello, {now}")
            last_check = now

    mqttc_close()


if __name__ == "__main__":
    app()



import json
import logging
import uuid

import paho.mqtt.client as mqtt

# get logger
log = logging.getLogger(__name__)

# global objects
_mqttc = None
_qos = 0
_count = 0


# Callbacks
def _on_connect(client, userdata, flags, rc, properties=None):
    if rc == 0:
        log.info("Connection successful")
        # Subscribing in on_connect() means that if we lose the connection and
        # reconnect then subscriptions will be renewed.
        client.subscribe("ysdtest/#")
    elif rc == 1:
        log.warning("Connection refused - incorrect protocol version")
    elif rc == 2:
        log.warning("Connection refused - invalid client identifier")
    elif rc == 3:
        log.warning("Connection refused - server unavailable")
    elif rc == 4:
        log.warning("Connection refused - bad username or password")
    elif rc == 5:
        log.warning("Connection refused - not authorised")
    else:
        log.warning("Unknown error")


def _on_disconnect(client, userdata, rc):
    log.info(f"disconnected. rc={rc}.")


def _on_message(client, obj, msg):
    if msg.retain == 0:
        log.info(f"Receive message ({msg.topic}): {msg.payload}")
    else:
        log.info(f"Receive retained message ({msg.topic}): {msg.payload}")


def _on_publish(client, obj, mid):
    log.debug("Sent.")


def _on_subscribe(client, obj, mid, granted_qos):
    log.debug("Subscribed.")


def _on_log(client, obj, level, string):
    log.debug(f"{level} | {string}")


def mqttc_setup(
        host: str, port=1883, username="", password="", qos=0, mqtt_debug=False
):
    try:
        # If you want to use a specific client id, use
        # mqttc = mqtt.Client("client-id")
        # but note that the client id must be unique on the broker. Leaving the client
        # id parameter empty will generate a random id for you.
        global _mqttc, _qos
        _qos = qos
        _mqttc = mqtt.Client(str(uuid.uuid4()), clean_session=(_qos == 0))
        # if usetls:
        #     if args.tls_version == "tlsv1.2":
        #         tlsVersion = ssl.PROTOCOL_TLSv1_2
        #     elif args.tls_version == "tlsv1.1":
        #         tlsVersion = ssl.PROTOCOL_TLSv1_1
        #     elif args.tls_version == "tlsv1":
        #         tlsVersion = ssl.PROTOCOL_TLSv1
        #     elif args.tls_version is None:
        #         tlsVersion = None
        #     else:
        #         print("Unknown TLS version - ignoring")
        #         tlsVersion = None
        #
        #     if not args.insecure:
        #         cert_required = ssl.CERT_REQUIRED
        #     else:
        #         cert_required = ssl.CERT_NONE
        #
        #     mqttc.tls_set(ca_certs=args.cacerts, certfile=None, keyfile=None, cert_reqs=cert_required, tls_version=tlsVersion)
        #
        #     if args.insecure:
        #         mqttc.tls_insecure_set(True)

        if username or password:
            _mqttc.username_pw_set(username, password)

        _mqttc.on_message = _on_message
        _mqttc.on_connect = _on_connect
        _mqttc.on_disconnect = _on_disconnect
        _mqttc.on_publish = _on_publish
        _mqttc.on_subscribe = _on_subscribe

        if mqtt_debug:
            _mqttc.on_log = _on_log

        # try to connect
        _mqttc.connect(host=host, port=port, keepalive=60)

        # start loop - let paho manage connection
        _mqttc.loop_start()

    except Exception as ex:
        log.error("mqtt connection error")
        raise ex


def mqttc_publish(topic, msg, qos=_qos, retain=False):
    global _mqttc, _count
    try:
        if _mqttc and _mqttc.is_connected():
            if not msg:
                msg = json.dumps({"count": f"{_count}"})
            log.info("Publishing: " + msg)
            pubinfo = _mqttc.publish(topic, msg, qos, retain)
            pubinfo.wait_for_publish()
            _count += 1
    except Exception as ex:
        log.error("mqtt publish error")
        raise ex


def mqttc_close():
    global _mqttc
    _mqttc.loop_stop()
    _mqttc.disconnect()



try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata
__version__ = importlib_metadata.version(__name__)

from .main import app
from .mqtt_task import mqttc_setup, mqttc_publish, mqttc_close



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['app_mqttc']

package_data = \
{'': ['*']}

install_requires = \
['paho-mqtt>=1.5.1,<2.0.0', 'typer[all]>=0.3.2,<0.4.0']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>=1.0,<2.0']}

entry_points = \
{'console_scripts': ['mqttc = app_mqttc:app']}

setup_kwargs = {
    'name': 'app-mqttc',
    'version': '0.1.2',
    'description': '',
    'long_description': '# app-mqttc\n\n',
    'author': 'Ying Shaodong',
    'author_email': 'helloysd@foxmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
